/*

    hackspacecon wand firmware

    initially written by true
    rgb programs and further hacking by 000000widow
  
  operation workflow:
  - at poweron, set up GPIO and used peripherals, then go into standby mode.
  - when button interrupt happens, wake up and process. if button is pushed,
    prepare to run RGB program.
  - enable the TCB0 periodic timer, which will interrupt every ~61Hz.
  - initialize the rgb program. idle the CPU.
  - timer will interrupt every ~61Hz and the program function will be called
    again, until prog function returns 0.
  - run the RGB program. if program returns 0, disable periodic timer, and
    wait for the next button event to wake us up and run a program again.
  
  todo:
  - test the code
  - actually process the RGB data in the sample rainbow puke program
  - add more programs
  - support incrementing or randomly selecting the next program on each button push
  - run TCB0 in standby mode to save a little more power
*/

#include <Arduino.h>
#include <avr/io.h>
#include <avr/interrupt.h>

#include "rgbled.h"



#include <tinyNeoPixel_Static.h>

tinyNeoPixel rgb = tinyNeoPixel(RGB_COUNT, PIN_PA1, NEO_GRB, rgbled);



uint8_t run_rgb_program = 0;



void setup() {
	// configure PA3 as falling edge interrupt input for button
	PORTA.DIRCLR = PIN3_bm;
    PORTA.PIN3CTRL = PORT_PULLUPEN_bm | PORT_ISC_BOTHEDGES_gc;

	// configure other hardware pins as appropriate
	pinMode(PIN_PA1, INPUT_PULLUP);	// unused
	pinMode(PIN_PA7, INPUT_PULLUP);	// unused

	digitalWrite(PIN_PA6, LOW);
	pinMode(PIN_PA6, OUTPUT);		// LED boost regulator enable

	digitalWrite(PIN_PA2, LOW);
	pinMode(PIN_PA2, OUTPUT);		// LED data

	// set up the RGB ~61Hz periodic timer
	conf_rgb_timer();
	
	// enable global interrupts (though they may already be enabled? fuck if I know)
	sei();
}

void loop() {
	switch (run_rgb_program) {
		case 1: {	// just started running a program
			digitalWrite(PIN_PA6, HIGH);	// enable LED power supply,
			delay(10);						// wait a moment for LEDs to stabilize,

			rgb_program[0](1);				// initialize the program,
			run_rgb_program++;				// and set to running mode.

			enable_rgb_timer();				// then start the RGB program timebase.

			// we can idle the CPU after running the program
			SLPCTRL.CTRLA = SLPCTRL_SMODE_IDLE_gc | SLPCTRL_SEN_bm;
			__asm("sleep");

			break;
		}
		case 2: {	// continuing to run a program
			// send updates to the led
			rgb.show();

			// then process the next program frame
			if (!rgb_program[0](0)) {
				// until the program says it's done
				run_rgb_program = 0;
				break;
			}

			// we can idle the CPU after running the program
			SLPCTRL.CTRLA = SLPCTRL_SMODE_IDLE_gc | SLPCTRL_SEN_bm;
			__asm("sleep");

			break;
		}
		default: {	// no longer running a program
			disable_rgb_timer();			// disable RGB program timer,
			digitalWrite(PIN_PA6, LOW);		// disable LED power supply,
			run_rgb_program = 0;			// and clear run_rgb_program.

			// finally, go to sleep in standby mode
			SLPCTRL.CTRLA = SLPCTRL_SMODE_STDBY_gc | SLPCTRL_SEN_bm;
			__asm("sleep");

			break;
		}
	}
}

// button interrupt
ISR(PORTA_PORT_vect)
{
    // reset the INTFLAGS - necessary on this series
    uint8_t intflags = PORTA.INTFLAGS;
    PORTA.INTFLAGS = intflags;

	// was our pin changed?
    if (intflags & PIN3_bm) {
		// is the pin low?
		if (!digitalRead(PIN_PA3)) {
			// start running a program if one isn't running already
        	if (!run_rgb_program) run_rgb_program = 1;
		}
    }
}